//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef CAPTURE_GNSS_TDCP_H_
#define CAPTURE_GNSS_TDCP_H_

//Wolf includes
#include "core/capture/capture_base.h"
#include "../feature/feature_gnss_tdcp.h"

//std includes
//

namespace wolf {
    
WOLF_PTR_TYPEDEFS(CaptureGnssTdcp);

class CaptureGnssTdcp : public CaptureBase
{
    protected:
        Eigen::Vector3d data_; ///< Displacement in ECEF coordinates.
        Eigen::Matrix3d data_covariance_; ///< Noise of the capture.
        FrameBasePtr    origin_frame_ptr_;
        TimeStamp ts_GPST_; ///< Time stamp in GPS time

    public:
        CaptureGnssTdcp(const TimeStamp& _ts,
                              SensorBasePtr _sensor_ptr,
                              const Eigen::Vector3d& _data,
                              const Eigen::Matrix3d& _data_covariance,
                              const FrameBasePtr& _origin_frame_ptr) :
            CaptureBase("CaptureGnssTdcp", _ts, _sensor_ptr),
            data_(_data),
            data_covariance_(_data_covariance),
            origin_frame_ptr_(_origin_frame_ptr)
        {};
        ~CaptureGnssTdcp() override{};
        const Eigen::Vector3d& getData() const;
        const Eigen::Matrix3d& getDataCovariance() const;
        void getDataAndCovariance(Eigen::Vector3d& data, Eigen::Matrix3d& data_cov) const;
        FrameBasePtr getOriginFrame() const;
        const TimeStamp& getGpsTimeStamp() const;
        void setGpsTimeStamp(const TimeStamp &_ts_GPST);
};

inline const Eigen::Vector3d& CaptureGnssTdcp::getData() const
{
    return data_;
}

inline const Eigen::Matrix3d& CaptureGnssTdcp::getDataCovariance() const
{
    return data_covariance_;
}

inline void CaptureGnssTdcp::getDataAndCovariance(Eigen::Vector3d& data, Eigen::Matrix3d& data_cov) const
{
    data = data_;
    data_cov = data_covariance_;
}

inline FrameBasePtr CaptureGnssTdcp::getOriginFrame() const
{
    return origin_frame_ptr_;
}

inline const wolf::TimeStamp& CaptureGnssTdcp::getGpsTimeStamp() const
{
    return ts_GPST_;
}

inline void CaptureGnssTdcp::setGpsTimeStamp(const TimeStamp &_ts_GPST)
{
    ts_GPST_ = _ts_GPST;
}

} //namespace wolf
#endif
