//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include "gnss/tree_manager/tree_manager_sliding_window_tdcp.h"

namespace wolf
{

TreeManagerSlidingWindowTdcp::TreeManagerSlidingWindowTdcp(ParamsTreeManagerSlidingWindowTdcpPtr _params) :
            TreeManagerSlidingWindow(_params),
            params_sw_sb_(_params)
{
    NodeBase::setType("TreeManagerSlidingWindowTdcp");
};

void TreeManagerSlidingWindowTdcp::keyFrameCallback(FrameBasePtr _key_frame)
{
    assert(getProblem() && "TreeManagerSlidingWindowTdcp::keyFrameCallback: problem not set.");

    // store first frame
    auto first_frame = getProblem()->getTrajectory()->getFirstFrame();

    // call base sliding window tree manager
    TreeManagerSlidingWindow::keyFrameCallback(_key_frame);

    // if first frame was removed, activate all factors of new first frame
    if (first_frame != getProblem()->getTrajectory()->getFirstFrame())
    {
        assert(first_frame->isRemoving());
        for (auto fac : getProblem()->getTrajectory()->getFirstFrame()->getConstrainedByList())
            if (fac and not fac->isRemoving() and
                (fac->getType() == "FactorGnssTdcp" or
                 fac->getType() == "FactorGnssTdcp2d" or
                 fac->getType() == "FactorGnssTdcp3d" or
                 fac->getType() == "FactorGnssTdcpBatch"))
                fac->setStatus(FAC_ACTIVE);
    }
}

} /* namespace wolf */

// Register in the FactoryTreeManager
#include "core/tree_manager/factory_tree_manager.h"
namespace wolf {
WOLF_REGISTER_TREE_MANAGER(TreeManagerSlidingWindowTdcp);
WOLF_REGISTER_TREE_MANAGER_AUTO(TreeManagerSlidingWindowTdcp);
} // namespace wolf

